/*
 * daemon.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include "openft.h"

#include "netorg.h"
#include "search.h"

/*****************************************************************************/

/**/extern Connection *ft_self;
/**/extern unsigned long search_unique_id;

/*****************************************************************************/

typedef void (*EventCallback) (Protocol *p, Connection *c, Dataset *event);
#define EV_HANDLER(func) static void ft_##func (Protocol *p, Connection *c, Dataset *event)

EV_HANDLER (daemon_promote);
EV_HANDLER (daemon_search);
EV_HANDLER (daemon_share);
EV_HANDLER (daemon_stats);

/*****************************************************************************/

struct _daemon_event
{
	char *name;
	EventCallback cb;
}
daemon_events[] =
{
	{ "promote",    ft_daemon_promote },
	{ "search",     ft_daemon_search  },
	{ "share",      ft_daemon_share   },
	{ "stats",      ft_daemon_stats   },
	{ NULL,         NULL              },
};

/*****************************************************************************/

void ft_daemon_callback (Protocol *p, Connection *c, Dataset *event)
{
	struct _daemon_event *ptr;
	char *ev_name;

	ev_name = dataset_lookup (event, "head");

	for (ptr = daemon_events; ptr->name; ptr++)
	{
		if (strcasecmp (ptr->name, ev_name))
			continue;

		/* found the appropriate event */
		(*ptr->cb) (p, c, event);

		return;
	}

	printf ("no handler found for %s\n", ev_name);
}

/*****************************************************************************/

static Connection *broadcast_status (Connection *c, Node *node, void *data)
{
	ft_node_send (c, FT_CLASS_RESPONSE, "%hu", NODE (ft_self)->class);

	return NULL;
}

EV_HANDLER (daemon_promote)
{
	node_class_add (ft_self, NODE_SEARCH);

	conn_foreach ((ConnForeachFunc) broadcast_status, NULL,
	              NODE_USER, NODE_CONNECTED, 0);
}

/*****************************************************************************/

#if 0
EV_HANDLER (daemon_connect)
{
	char *ip, *port;

	ip   = dataset_lookup (event, "ip");
	port = dataset_lookup (event, "port");

	if (!ip || !port)
		return;

	ft_connect (p, ip, atoi (port));
}
#endif

/*****************************************************************************/

static Connection *broadcast_search (Connection *c, Node *node, Search *search)
{
	ft_node_send (c, FT_SEARCH_REQUEST, "%lu%hu%s",
	              search->id, (unsigned short) search->type, search->query);

	search->ref++;

	return NULL;
}

EV_HANDLER (daemon_search)
{
	Search *search;
	int     search_type;
	char   *query;
	char   *type;

	query = dataset_lookup (event, "query");
	type  = dataset_lookup (event, "type");

	if (!query)
		return;

	if (!type)
		search_type = SEARCH_FILENAME;
	else
	{
		if (!strcasecmp (type, "md5"))
			search_type = SEARCH_MD5;
		else if (!strcasecmp (type, "user"))
			search_type = SEARCH_HOST;
	}

	/* create the local OpenFT search object */
	search = search_new ((IFEventID) dataset_lookup (event, "id"),
	                     search_type, query);

	if (NODE (ft_self)->class & NODE_SEARCH)
	{
		List *ptr;

		printf ("\tmyself\n");

		for (ptr = ft_search (search_type, query); ptr; ptr = ptr->next)
		{
			FileShare *file = ptr->data;

			/* ignore local shares */
			if (file->host != 0)
				search_reply (search->id, ft_self, file);
		}

#if 0
		/* this is commented out because it simply deals w/ ref counting,
		 * and that is not a concern here */
		search_reply (search->id, ft_self, NULL);
#endif
	}

	/* local connections are sort of an exception.  they do not count as a
	 * reference as it completes in the loop above */
	search->ref = 0;

	conn_foreach ((ConnForeachFunc) broadcast_search, search,
	              NODE_SEARCH, NODE_CONNECTED, 0);

	printf ("search->ref = %i\n", search->ref);

	if (!search->ref)
	{
		/* basically call search_remove, but handle it more naturally */
		search->ref = 1;
		search_reply (search->id, ft_self, NULL);
	}
}

/*****************************************************************************/

static Connection *share_sync (Connection *c, Node *node, void *data)
{
	ft_node_send (c, FT_SHARE_REQUEST, NULL);
	share_local_submit (c->protocol, c);

	return NULL;
}

EV_HANDLER (daemon_share)
{
	char *action;

	if (!(action = dataset_lookup (event, "action")))
		return;

	if (strcmp (action, "sync"))
		return;

	/* share_local_rebuild (); */

	/* sync == re-read ~/.giFT/shares */
	share_local_import ();
	conn_foreach ((ConnForeachFunc) share_sync, NULL,
	              NODE_SEARCH, NODE_CONNECTED, 0);
}

/*****************************************************************************/

static Connection *stats (Connection *c, Node *node, Connection *dc)
{
	interface_send (dc, "NODE",
	                "HOST=s",   net_ip_str (node->ip),
	                "PORT=i",   node->port,
	                "CLASS=s",  node_class_str (node),
	                "USERS=i",  node->users,
	                "SHARES=i", node->shares,
	                "MEGS=i",   node->megs,
	                NULL);

	return NULL;
}

EV_HANDLER (daemon_stats)
{
	stats (ft_self, NODE (ft_self), c);
	conn_foreach ((ConnForeachFunc) stats, c, NODE_USER, NODE_CONNECTED, 0);

	interface_close (c);
}
