/*
 * html.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>

#include "openft.h"

#include "netorg.h"

#include "http.h"
#include "html.h"

#include "parse.h"

/*****************************************************************************/

/* #define IMAGE_HOST "http://www.theproject.nu/~stoned/gift" */
#define IMAGE_HOST "/OpenFT"

static time_t     nodepage_mtime = 0;

/*****************************************************************************/

/* strip HTML */
char *html_strip (char *html)
{
	char *head, *stripped, *begin, *end;

	if (!html)
		return NULL;

	/* not very effecient, but clean */
	head = stripped = strdup (html);

	while ((begin = strchr (head, '<')))
	{
		/* a '<' was found, look for the '>' */
		for (end = begin + 1; *end && *end != '>'; end++);

		if (*end)
			end++;

		/* shift the string back */
		strmove (begin, end);

		head = begin;
	}

	return stripped;
}

/*****************************************************************************/
/* url decode/encode helpers */

static int oct_value_from_hex (char hex_char)
{
	if (!isxdigit (hex_char))
		return 0;

	if (hex_char >= '0' && hex_char <= '9')
		return (hex_char - '0');

	hex_char = toupper (hex_char);

	return ((hex_char - 'A') + 10);
}

char *url_decode (char *encoded)
{
	char *decoded, *ptr;

	if (!encoded)
		return NULL;

	/* make sure we are using our own memory here ... */
	ptr = strdup (encoded);

	/* save the head */
	decoded = ptr;

	/* convert '+' -> ' ' and %2x -> char value */
	while (*ptr)
	{
		switch (*ptr)
		{
		 case '+':
			*ptr = ' ';
			break;
		 case '%':
			if (isxdigit (ptr[1]) && isxdigit (ptr[2]))
			{
				int oct_val;

				oct_val =  oct_value_from_hex (ptr[1]) * 16;
				oct_val += oct_value_from_hex (ptr[2]);

				*ptr = (char) oct_val;

				strmove (ptr + 1, ptr + 3);
			}
			break;
		 default:
			break;
		}

		ptr++;
	}

	return decoded;
}

static char *url_encode_char (char *stream, char c)
{
	*stream++ = '%';

	sprintf (stream, "%02x", (unsigned int) c);

	return stream + 2;
}

char *url_encode (char *decoded)
{
	char *encoded, *ptr;

	if (!decoded)
		return NULL;

	/* allocate a large enough buffer for all c ases */
	encoded = ptr = malloc ((strlen (decoded) * 3) + 1);

	while (*decoded)
	{
		switch (*decoded)
		{
		 case ' ':
/*			*ptr++ = '+';
			break;*/
		 case '!': /* any more? */
		 case '?':
		 case '+':
		 case '%':
		 case '&':
		 case ':':
		 case '=':
		 case '(':
		 case ')':
		 case '[':
		 case ']':
			ptr = url_encode_char (ptr, *decoded);
			break;
		 default:
			*ptr++ = *decoded;
			break;
		}

		decoded++;
	}

	*ptr = 0;

	return encoded;
}

/*****************************************************************************/

void html_update_nodepage ()
{
	nodepage_mtime = time (NULL);
}

/*****************************************************************************/

/* unoptimized crap */
static Connection *build_nodelist (Connection *c, Node *node, FILE *f)
{
	if (node->state != NODE_CONNECTED)
		return NULL;

	fprintf (f, "<TR><TD>");

	/* add the node link */
	if (!node->firewalled)
	{
		fprintf (f, "<A HREF=\"http://%s:%hu/\">",
				 net_ip_str (node->ip), node->http_port);
	}

	fprintf (f, "%s", net_ip_str (node->ip));

	if (!node->firewalled)
		fprintf (f, ":%hu</A>", node->http_port);

	fprintf (f, "</TD><TD>%s%s</TD></TR>\n",
			 node_class_str (node),
			 (node->firewalled ? "[Firewalled]" : ""));

	return NULL;
}

/* also, unoptimized crap. :) */
static int build_shares (unsigned long key, FileShare *file, FILE *f)
{
	if (!file)
		return FALSE;

	fprintf (f, "<TR><TD>%lu</TD><TD><A HREF=\"%s\">%s</A></TD></TR>\n",
	         file->size, url_encode (file->file), file->file);

	return TRUE;
}

char *html_page_index (char *file)
{
	FILE *f;
	struct stat st;

	if (stat (file, &st))
	{
		nodepage_mtime = time (NULL);
		st.st_mtime = 0;
	}

	/* nodepage_mtime is updated anytime the file should be rebuilt */
	if (nodepage_mtime == st.st_mtime)
		return file;

	if (!(f = fopen (file, "w")))
		return NULL;

	trace ();

	/* write the header */
	fprintf (f,
			 "<HTML>\n"
			 " <HEAD>\n"
			 "  <TITLE>giFT: OpenFT</TITLE>\n"
			 "  <LINK REL=STYLESHEET TYPE=\"TEXT/CSS\" HREF=\"" IMAGE_HOST "/other.css\">\n"
			 " </HEAD>\n"
			 " <BODY BGCOLOR=#788896>\n"
			 " <TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 WIDTH=764 ALIGN=CENTER>\n"
			 "  <TR>\n"
			 "   <TD WIDTH=3><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=3 HEIGHT=1></TD>\n"
			 "   <TD WIDTH=703><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=703 HEIGHT=1></TD>\n"
			 "   <TD WIDTH=3><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=3 HEIGHT=1></TD>\n"
			 "   <TD WIDTH=55><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=55 HEIGHT=1></TD>\n"
			 "  </TR>\n"
			 "  <TR>\n"
			 "   <TD COLSPAN=4><IMG SRC=\"" IMAGE_HOST "/top.jpg\" WIDTH=764 HEIGHT=134></TD>\n"
			 "  </TR>\n"
			 "  <TR>\n"
			 "   <TD BACKGROUND=\"" IMAGE_HOST "/black.gif\"><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=3 HEIGHT=1></TD>\n"
			 "   <TD BACKGROUND=\"" IMAGE_HOST "/white.gif\">\n"
			 "    <TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 WIDTH=703 ALIGN=center>\n"
			 "     <TR>\n"
			 "      <TD>\n"
			 "<!-- BEGIN OPENFT BODY -->\n");

	/* separated for better tracking of the %s */
	fprintf (f, "<H1>%s %s (%s)</H1>\n", PACKAGE, VERSION, GIFT_CVSID);
	fprintf (f, "<H1>Node Status: %s</H1>\n", node_class_str (NULL));

	fprintf (f,
	         "<H2>Node List</H2>\n"
	         "<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2>\n"
	         "<TR><TD><B>Node</B></TD><TD><B>Classification</B></TD></TR>\n");

	/* insert nodelist here */
	conn_foreach ((ConnForeachFunc) build_nodelist, f,
	              NODE_USER, NODE_CONNECTED, 0);

	fprintf (f,
	         "</TABLE>\n"
	         "<H2>Shares</H2>\n"
	         "<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=2>\n"
	         "<TR><TD><B>Filesize</B></TD><TD><B>Filename</B></TD></TR>\n");

	/* insert shares list here */
	share_local_foreach ((HashFunc) build_shares, f);

	fprintf (f,
	         "</TABLE>\n"
	         "<!-- END OPENFT BODY -->\n"
	         "      </TD>\n"
	         "     </TR>\n"
	         "    </TABLE>\n"
	         "   </TD>\n"
	         "   <TD BACKGROUND=\"" IMAGE_HOST "/black.gif\"><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=3 HEIGHT=1></TD>\n"
	         "   <TD BACKGROUND=\"" IMAGE_HOST "/bg.gif\" VALIGN=top ROWSPAN=2><IMG SRC=\"" IMAGE_HOST "/upperrightcorner.gif\" WIDTH=55 HEIGHT=4></TD>\n"
	         "  </TR>\n"
	         "  <TD>\n"
	         "   <TD BACKGROUND=\"" IMAGE_HOST "/black.gif\" COLSPAN=2><IMG SRC=\"" IMAGE_HOST "/spacer.gif\" WIDTH=3 HEIGHT=3></TD>\n"
	         "  </TR>\n"
	         " </TABLE>\n"
	         " </BODY>\n"
	         "</HTML>\n");

	fclose (f);

	/* modify the global mtime */
	stat (file, &st);
	nodepage_mtime = st.st_mtime;

	return file;
}

/*****************************************************************************/

char *html_page_redirect (char *page)
{
	FILE *f;

	if (!(f = fopen (page, "w")))
		return NULL;

	fprintf (f, "<html><body>Redirection successful</body></html>\n");
	fclose (f);

	return page;
}
