/*
 * http.h
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __HTTP_H
#define __HTTP_H

/*****************************************************************************/

typedef void (*HTTPTransferCB) (Connection *c, Chunk *chunk, char *data,
								size_t len);

typedef struct _http_transfer
{
	Chunk         *chunk;

	/* help give the stats back to the daemon */
	HTTPTransferCB callback;
	Protocol      *protocol;

	FILE          *f;

	/* header information along the way */
	Dataset       *header;

	unsigned long  ip;
	unsigned short port;
	char          *filename;
	char          *encoded;

	unsigned long  transmit;

	/* Range: request */
	unsigned long  range_start;
	unsigned long  range_stop;

	/* many transfers can be with regards to nodepages, information, etc
	 * and will NOT be counted as an upload to the daemon! */
	int file_transfer;
} HTTPTransfer;

/*****************************************************************************/

HTTPTransfer *http_transfer_new (HTTPTransferCB cb, Protocol *p,
								 unsigned long ip, unsigned short port,
								 char *filename, Chunk *chunk);

void http_handle_incoming (Protocol *p, Connection *c);
int  http_push_file       (HTTPTransfer *transfer);
int  http_pull_file       (HTTPTransfer *transfer);
void http_cancel          (Chunk *chunk);

/*****************************************************************************/

#endif /* __HTTP_H */
