/*
 * mime.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <ctype.h>

#include "openft.h"

#include "file.h"
#include "parse.h"

/*****************************************************************************/

struct _mime_type
{
	char *type;
	char *extensions;
};

List *mime_types = NULL;

/*****************************************************************************/

static void load_types ()
{
	FILE *f;
	char *buf = NULL;
	char *ptr;

	if (!(f = fopen (DATA_DIR "/mime.types", "r")))
	{
		GIFT_ERROR (("failed to open " DATA_DIR "/mime.types"));
		return;
	}

	while (file_read_line (f, &buf))
	{
		struct _mime_type *mime_info;
		char *type;

		ptr = buf;

		trim_whitespace (ptr);

		if (*ptr == '#')
			continue;

		type = string_sep_set (&ptr, " \t");

		if (!ptr || !ptr[0])
			continue;

		while (isspace (*ptr))
			ptr++;

		mime_info = malloc (sizeof (struct _mime_type));
		mime_info->type       = STRDUP (type);
		mime_info->extensions = STRDUP (ptr);

		mime_types = list_append (mime_types, mime_info);
	}

	fclose (f);
}

/*****************************************************************************/

/* locates the appropriate mime type based on extension */
static char *mime_type (char *file)
{
	struct _mime_type *mime_info;
	char *type = NULL;
	char *ext;
	List *ptr;

	if (!mime_types)
		load_types ();

	if ((ext = strrchr (file, '.')))
		ext++;
	else
		type = "text/plain";

	for (ptr = mime_types; ptr; ptr = list_next (ptr))
	{
		char *exts, *exts0;
		char *ext_cmp;

		if (type)
			break;

		mime_info = ptr->data;

		exts0 = exts = STRDUP (mime_info->extensions);

		while ((ext_cmp = string_sep (&exts, " ")))
		{
			if (!strcasecmp (ext_cmp, ext))
				type = mime_info->type;
		}

		free (exts0);
	}

	if (!type)
		type = "text/plain";

	return type;
}

/* TODO - skip id3 tags and headers */
static unsigned long mime_size (char *file, char *mime)
{
	return file_size (file);
}

/*****************************************************************************/

FILE *mime_open (char *file, char *mode, char **mime, unsigned long *size)
{
	FILE *f;

	assert (file);

	if (mime)
		*mime = mime_type (file);

	if (size)
		*size = mime_size (file, *mime);

	f = fopen (file, mode);

	return f;
}
