/*
 * node.h
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __NODE_H__
#define __NODE_H__

#include "queue.h"

/**************************************************************************/

typedef struct
{
	/* last known connected timestamp */
	unsigned long  vitality;

	/* this node connected to us...temporary state flag, not important :) */
	int            incoming;

	/* stats */
	unsigned long  users;
	unsigned long  shares;
	unsigned long  megs;

	/* information used to get back to the given host */
	unsigned long  ip;
	unsigned short port;
	unsigned short http_port;
	int            firewalled;

	/* current state of the given connection */
	enum
	{
		NODE_DISCONNECTED  = 0x00, /* functionless node */
		NODE_CONNECTING    = 0x01, /* pending */
		NODE_CONNECTED     = 0x02, /* first packet is seen */
		NODE_IDLE          = 0x04, /* after handshake */
	} state;

	/* node classification that this connection is communicating with
	 * (bitwise OR) */
	enum
	{
		NODE_NONE   = 0x00,
		NODE_USER   = 0x01,
		NODE_SEARCH = 0x02,
		NODE_INDEX  = 0x04,

		NODE_CHILD  = 0x08         /* node is sharing files to 'self' */
	} class;

	/* file descriptor */
	int            fd;

	/* reference to our select loop place */
	int            input;
} Node;

#define NODE(c) ((Node *)c->data)

/**************************************************************************/

int ft_node_send          (Connection *c, unsigned short ft_command,
                           char *fmt, ...);
int ft_node_send_indirect (unsigned long deliver_to, unsigned short ft_command,
                           char *fmt, ...);

void ft_node_connection (Protocol *p, Connection *c);

Connection *node_add_connection (Protocol *p, int fd);
void node_remove_connection     (Protocol *p, Connection *c);

int node_maintain_links (Protocol *p);

void node_state_set    (Connection *c, unsigned short state);
void node_class_set    (Connection *c, unsigned short class);
void node_class_add    (Connection *c, unsigned short class);
void node_class_remove (Connection *c, unsigned short class);
void node_conn_set     (Connection *c, unsigned long ip, signed long port,
                        signed long http_port);

/**************************************************************************/

#endif /* __NODE_H__ */
