/*
 * utils.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <ctype.h>

#include "openft.h"

#include "netorg.h"

#include "http.h"

/*****************************************************************************/

/**/extern Connection *ft_self;

/*****************************************************************************/

off_t file_size (char *file)
{
	struct stat st;

	/* uh oh. */
	if (stat (file, &st))
		return -1;

	return st.st_size;
}

/*****************************************************************************/

/*
 * Network handling routines -- basic node helpers
 */

int ft_connect (Protocol *p, char *ip, unsigned short port)
{
	Connection *c;
	int fd;

	if (port == 0)
		return -1;

	ip = strdup (ip);

	/* due to the nature of this network, multiple connections may be tried
	 * often... do not allow that to happen */
	if (!conn_auth (ip, port))
	{
		free (ip);
		return -1;
	}

	/* ok we're not connected to this host already ... proceed */
	fd = net_connect (ip, port);

	c = node_add_connection (p, fd);

	/* ip and port were likely set wrong if this connection fucked up, so make
	 * sure they're set as what ft_connect wanted them to be */
	node_conn_set (c, inet_addr (ip), port, -1);

	if (fd >= 0)
	{
		node_state_set (c, NODE_CONNECTING);

		input_add (p, c, INPUT_WRITE, (InputCallback) ft_handle_source);
	}
	else
		node_state_set (c, NODE_DISCONNECTED);

	free (ip);

	return fd;
}

int ft_reconnect (Protocol *p, Connection *c)
{
	c->fd = net_connect (net_ip_str (NODE (c)->ip), NODE (c)->port);
	NODE (c)->fd = c->fd;

	/* should already be in the ft_connections list */

	node_state_set (c, NODE_CONNECTING);

	input_add (p, c, INPUT_WRITE, (InputCallback) ft_handle_source);

	return c->fd;
}

/* connection verification */
static int ft_handle_verification (Protocol *p, Connection *c)
{
	/* make sure we got a valid connection */
	if (net_sock_error (c->fd))
	{
		node_remove_connection (p, c);

		return FALSE;
	}

	/* switch to a new cb */
	input_remove (c);
	input_add (p, c, INPUT_READ, (InputCallback) ft_node_connection);

	/* HANDSHAKE -- TODO - move this/clean up */

	/* if this was an incoming connection, gather information about its
	 * node */
	if (NODE (c)->incoming)
		ft_node_send (c, FT_NODEINFO_REQUEST, NULL);

	/* leech as much info from this node as possible... */
	ft_node_send (c, FT_CLASS_REQUEST, NULL);
	ft_node_send (c, FT_NODELIST_REQUEST, NULL);

	return TRUE;
}

/* incoming connections callback */
void ft_handle_incoming (Protocol *p, Connection *c)
{
	Connection *new_c;
	int fd;

	fd = net_accept (c->fd);

	/* check to make sure this connection already exists... */
	if (!conn_auth (net_peer_ip (fd), 1215))
		return;

	/* we're good, add it. */
	new_c = node_add_connection (p, fd);

	NODE (new_c)->incoming = TRUE;
	node_state_set (new_c, NODE_CONNECTING);
	node_class_set (new_c, NODE_NONE);

	/* throw it into the select loop */
	input_add (p, new_c, INPUT_WRITE, (InputCallback) ft_handle_verification);
}

/* outgoing connections callback */
void ft_handle_source (Protocol *p, Connection *c)
{
	if (!ft_handle_verification (p, c))
		return;
}

/*****************************************************************************/

char *node_class_str (Node *node)
{
	char *str;

	if (!node)
		node = NODE (ft_self);

	if (node->class & NODE_INDEX)
		str = "INDEX";
	else if (node->class & NODE_SEARCH)
		str = "SEARCH";
	else if (node->class & NODE_CHILD)
		str = "CHILD";
	else if (node->class & NODE_USER)
		str = "USER";
	else
		str = "NONE";

	return str;
}
