/*
 * hash.h
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __HASH_H
#define __HASH_H

#include "list.h"

/*****************************************************************************/

typedef int (*HashFunc) (unsigned long key, void *value, void *udata);

typedef struct _hash_node
{
	unsigned long  key;
	void          *value;

	struct _hash_node *next;
} HashNode;

typedef struct _hash_table
{
	size_t        size;
	unsigned long items;
	int           frozen;

	HashNode    **nodes;
} HashTable;

/*****************************************************************************/

/* wrappers for a more intrinsic interface as a data set */
#define Dataset HashTable
#define dataset_insert(dataset,key,value) \
	if(!dataset)dataset=hash_table_new(); \
	hash_table_insert(dataset,hash_string(key),(void*)value)
#define dataset_lookup(dataset,key) \
	hash_table_lookup(dataset,hash_string(key))
#define dataset_remove(dataset,key) \
	hash_table_remove(dataset,hash_string(key))
#define dataset_clear(dataset) \
	hash_table_destroy(dataset); dataset = NULL
#define dataset_clear_free(dataset) \
	hash_table_destroy_free(dataset); dataset = NULL

/*****************************************************************************/

HashTable     *hash_table_new     ();
void           hash_table_destroy (HashTable *table);
void           hash_table_destroy_free (HashTable *table);
List          *hash_flatten       (HashTable *table);
void          *hash_table_lookup  (HashTable *table, unsigned long key);
unsigned long  hash_string        (char *str);
void           hash_table_insert  (HashTable *table, unsigned long key,
                                   void *value);
void           hash_table_remove  (HashTable *table, unsigned long key);

void hash_table_foreach        (HashTable *table, HashFunc func, void *udata);
int  hash_table_foreach_remove (HashTable *table, HashFunc func, void *udata);

/*****************************************************************************/

#endif /* __HASH_H */
