/*
 * list.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <stdio.h>
#include <stdlib.h>

#include <assert.h>

#include "list.h"

/*****************************************************************************/
List *list_last (List *list)
{
	return list->last;
}

int list_length (List *list)
{
	int len = 0;

	while (list)
	{
		len++;
		list = list->next;
	}

	return len;
}

static List *list_new_entry (List *head, void *data)
{
	List *list_entry;

	list_entry = malloc (sizeof (List));

	list_entry->data = data;

	list_entry->next = NULL;
	list_entry->last = head ? head->last : list_entry;

	return list_entry;
}

/*****************************************************************************/
List *list_append (List *list, void *data)
{
	List *list_entry;
	List *last;

	list_entry = list_new_entry (list, data);

	if (!list)
		return list_entry;

	last = list_last (list);

	last->next = list_entry;
	list->last = list_entry;

	return list;
}

List *list_prepend (List *list, void *data)
{
	List *list_entry;

	list_entry = list_new_entry (list, data);

	if (!list)
		return list_entry;

	list_entry->next = list;

	return list_entry;
}

/*****************************************************************************/

#if 0
List *list_link_append (List *list, List *link)
{
	List *last;

	if (!list)
		return link;

	if (!link)
		return list;

	assert (link->last);
	assert (list->last);

	list->last->next = link;
	list->last = link->last;

	return list;
}

List *list_link_prepend (List *list, List *link)
{
	if (!list)
		return link;

	if (!link)
		return list;

	assert (link->last);
	assert (list->last);

	link->last->next = list;
	link->last = list->last;

	return link;
}
#endif

/*****************************************************************************/
List *list_copy (List *list)
{
	List *new_list = NULL, *ptr;

	for (ptr = list; ptr; ptr = ptr->next)
		new_list = list_append (new_list, ptr->data);

	return new_list;
}

/*****************************************************************************/
/* do _NOT_ use this function on anything but the head! */
List *list_shift (List *list, void **data)
{
	List *next;

	if (!list)
	{
		if (data)
			*data = NULL;

		return NULL;
	}

	if (data)
		*data = list->data;

	next = list->next;

	if (next)
		next->last = list->last;

	free (list);

	return next;
}

/*****************************************************************************/
List *list_remove (List *list, void *data)
{
	List *ptr, *prev = NULL, *last;

	if (!list)
		return NULL;

	last = list->last;
	if (!last)
	{
		printf ("** %s:%i: panic: invalid head passed!\n", __FILE__, __LINE__);
		return NULL;
	}

	for (ptr=list; ptr; ptr=ptr->next)
	{
		if (data == ptr->data)
		{
			if (prev)
				prev->next = ptr->next;

			/* head */
			if (ptr == list)
			{
				list = list->next;
				if (list)
					list->last = last;
			}
			/* tail */
			else if (ptr == last)
				list->last = prev ? prev : list;

			free (ptr);

			break;
		}
		prev = ptr;
	}

	return list;
}

/*****************************************************************************/
List *list_find (List *list, void *data)
{
	List *ptr;

	for (ptr=list; ptr; ptr=ptr->next)
	{
		if (data == ptr->data)
			return ptr;
	}

	return NULL;
}

/*****************************************************************************/
List *list_free (List *list)
{
	List *ptr = list, *next;

	while (ptr)
	{
		next = ptr->next;
		free (ptr);
		ptr = next;
	}

	return NULL;
}

/*****************************************************************************/
List *list_find_custom (List *list, void *data, CompareFunc func)
{
	if (!func)
		return NULL;

	/* TODO - verify this is correct. */
	for (; list; list = list_next (list))
		if (func (list->data, data) == 0)
			return list;

	return NULL;
}

/*****************************************************************************/

List *list_remove_custom (List *list, void *data, CompareFunc func)
{
   List *original = list;

   if (!func)
	 return (List *) NULL;

   for(; list; list = list_next (list))
	 if (func (list->data, data) == 0)
	   original = list_remove(original, list->data);

   return original;
}

/*****************************************************************************/
void *list_nth_data (List *list, int nth)
{
	if (nth < 0)
		return (void *) NULL;

	/* TODO - optimize */
	for (; list && nth; list = list_next (list), nth--)
		;

	if (list)
		return list->data;

	return (void *)NULL;
}
