/*
 * queue.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include "gift.h"

#include "network.h"
#include "event.h"
#include "queue.h"

/*****************************************************************************/

static void queue_write_default (Connection *c, char *data, void *udata)
{
	unsigned long len;

	if (!data)
		return;

	len = (udata ? (unsigned long) udata : strlen (data));

	send (c->fd, data, len, 0);
}

static void queue_destroy_default (Connection *c, char *data, void *udata)
{
	free (data);
}

/*****************************************************************************/

static void queue_write (Protocol *p, Connection *c)
{
	WriteQueue *queue;
	void       *data;

	if (!c->write_queue)
	{
		queue_free (c);
		/* input_remove_full (c, INPUT_WRITE, queue_write); */
		return;
	}

	queue = c->write_queue->data;

#if 0
	if (c->last_write)
	{
		if (c->last_write->func != queue->func)
			(*(c->last_write->func)) (c, NULL, queue->udata);
	}
#endif

	queue->buffer = list_shift (queue->buffer, &data);
	(*queue->func) (c, data, queue->udata);
	(*queue->destroy_func) (c, data, queue->udata);

	/* no more data left with this WriteQueue element, free it and move to
	 * the next */
	if (!data)
	{
		free (queue);
		c->write_queue = list_shift (c->write_queue, NULL);
	}
}

void queue_add_single (Connection *c, QueueWriteFunc func,
                       QueueWriteFunc destroy_func, void *write_data,
                       void *data)
{
	List *list;

	list = list_prepend (NULL, write_data);

	queue_add (c, func, destroy_func, list, data);
}

/* TODO - come up with a way to optimize this */
void queue_add (Connection *c, QueueWriteFunc func, QueueWriteFunc destroy_func,
                List *buffer, void *data)
{
	WriteQueue *queue;

	if (!func)
		func = (QueueWriteFunc) queue_write_default;

	if (!destroy_func)
		destroy_func = (QueueWriteFunc) queue_destroy_default;

	if (!buffer)
	{
		(*func) (c, NULL, data);
		return;
	}

	if (!c->write_queue)
		input_add (c->protocol, c, INPUT_WRITE, queue_write);

	queue = malloc (sizeof (WriteQueue));

	queue->c            = c;
	queue->func         = func;
	queue->destroy_func = destroy_func;
	queue->buffer       = buffer;
	queue->udata        = data;

	c->write_queue = list_append (c->write_queue, queue);

	/* list_free (head); */
}

void queue_flush (Connection *c)
{
	while (c->write_queue)
		queue_write (c->protocol, c);
}

void queue_free (Connection *c)
{
	WriteQueue *queue;
	List       *ptr;
	List       *next;

#if 0
	/* get rid of the last writequeue seen */
	if (c->last_write)
	{
		(*c->last_write->func) (c, NULL, c->last_write->udata);
		free (c->last_write);
		c->last_write = NULL;
	}
#endif

	input_remove_full (c, INPUT_WRITE, queue_write);

	if (!c->write_queue)
		return;

	/* destroy all data associated w/ the queue...very messy */
	for (ptr = c->write_queue; ptr; )
	{
		List *elem;
		List *elem_next;

		next  = list_next (ptr);
		queue = ptr->data;

		for (elem = queue->buffer; elem; )
		{
			elem_next = list_next (elem);

			if (queue->destroy_func)
				(*queue->destroy_func) (queue->c, elem->data, queue->udata);

			free (elem);

			elem = elem_next;
		}

		(*queue->func) (c, NULL, queue->udata);
		free (ptr);

		ptr = next;
	}

	c->write_queue = NULL;
}
