/*
 * transfer.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include "gift.h"

#include "transfer.h"
#include "event.h"

/*****************************************************************************/

/* laziness.  this should be typedef'd */
void download_write (Chunk *chunk, char *segment, size_t len);
void upload_write   (Chunk *chunk, char *segment, size_t len);

/*****************************************************************************/

Transfer *transfer_new (TransferType direction,
                        char *filename, unsigned long size)
{
	Transfer *transfer;

	if (!(transfer = malloc (sizeof (Transfer))))
		return NULL;

	memset (transfer, 0, sizeof (Transfer));

	transfer->type     = direction;

	transfer->filename = STRDUP (filename);
	transfer->transmit = 0;
	transfer->total    = size;
	transfer->max_seek = 0;

	transfer->timer    = -1;

	return transfer;
}

void transfer_free (Transfer *transfer)
{
	List *ptr;

	assert (transfer);

	for (ptr = transfer->chunks; ptr; ptr = list_next (ptr))
	{
		chunk_free (ptr->data);
	}

	list_free (transfer->chunks);

	free (transfer->filename);
	free (transfer->path);

	if (transfer->f)
		fclose (transfer->f);

	timer_remove (transfer->timer);
	if_event_remove (transfer->id);

	free (transfer);
}

/*****************************************************************************/

Chunk *chunk_new (Transfer *transfer, char *source, size_t start, size_t stop)
{
	Protocol *p;
	char *protocol;
	char *ptr;
	Chunk *chunk;

	protocol = STRDUP (source);

	if ((ptr = strchr (protocol, ':')))
	{
		*ptr = 0;
		p = protocol_find (protocol);
	}

	free (protocol);

	if (!p)
		return NULL;

	if (!(chunk = malloc (sizeof (Chunk))))
		return NULL;

	memset (chunk, 0, sizeof (Chunk));

	chunk->transfer = transfer;
	chunk->p        = p;
	chunk->source   = STRDUP (source);

	chunk->transmit = 0;
	chunk->start    = start;
	chunk->stop     = stop;
	chunk->active   = TRUE;

	transfer->chunks = list_append (transfer->chunks, chunk);

	return chunk;
}

void chunk_free (Chunk *chunk)
{
	assert (chunk);

	free (chunk->source);
	free (chunk);
}

void chunk_write (Chunk *chunk, char *segment, size_t len)
{
	Transfer *transfer = chunk->transfer;

	assert (chunk);
	assert (transfer);

	switch (transfer->type)
	{
	 case TRANSFER_DOWNLOAD:
		download_write (chunk, segment, len);
		break;
	 case TRANSFER_UPLOAD:
		upload_write (chunk, segment, len);
		break;
	 default:
		GIFT_ERROR (("unknown transfer type %i", transfer->type));
		break;
	}
}

void chunk_cancel (Chunk *chunk)
{
	Transfer *transfer = chunk->transfer;

	trace ();

	if (!chunk->active)
		return;

	chunk->active = FALSE;

	if (transfer->type == TRANSFER_DOWNLOAD)
		chunk->p->download (chunk);
}
