/*
 * transfer.h
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __TRANSFER_H
#define __TRANSFER_H

/*****************************************************************************/

typedef enum _transfer_type
{
	TRANSFER_DOWNLOAD,
	TRANSFER_UPLOAD
} TransferType;

typedef struct _transfer
{
	IFEventID     id;

	TransferType  type;     /* the type of the transfer */
	char         *filename; /* the name of the file */
	char         *path;     /* fully qualified local path to this file */

	int           active;   /* inactive when first created with 0 sources */

	unsigned long transmit;  /* total received */
	unsigned long total;     /* total filesize to download */
	unsigned long max_seek;  /* current filesize according to the filesystem */

	List         *chunks;   /* partitioned download segments */

	int           timer;    /* logs the download progress in the resume file */

	FILE         *f;        /* read/write file descriptor */
} Transfer;

/*****************************************************************************/

typedef struct _chunk
{
	Transfer     *transfer;
	Protocol     *p;
	char         *source;

	int           active;    /* inactive chunks have been divided, but no
							  * source is actively downloading */

	unsigned long transmit;  /* currently received */
	unsigned long start;
	unsigned long stop;
} Chunk;

/*****************************************************************************/

Transfer  *transfer_new  (TransferType direction, char *filename,
                          unsigned long size);
void       transfer_free (Transfer *transfer);

/*****************************************************************************/

Chunk     *chunk_new     (Transfer *transfer, char *source,
                          size_t start, size_t stop);
void       chunk_write   (Chunk *chunk, char *segment, size_t len);
void       chunk_free    (Chunk *chunk);
void       chunk_cancel  (Chunk *chunk);

/*****************************************************************************/

#endif /* __TRANSFER_H */
