/*
 * download.c
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include "gift-fe.h"

#include "menu.h"
#include "share.h"
#include "download.h"
#include "daemon.h"

/* download stats interval (in seconds) */
#define STATS_INTERVAL 1

/*****************************************************************************/

int download_insert (GtkWidget *dl_list, SharedFile *shr)
{
	Transfer     *transfer;
	char         *add_id[3];
	GtkCTreeNode *new_node;
	GtkCTreeNode *insert_node;

	/* setup the transfer object ... */
	transfer = OBJ_NEW (Transfer);

	share_dup (SHARE (transfer), shr);
	shr = SHARE (transfer);

	/* insert into the tree with the appropriate outsourced organization */
	insert_node = find_share_node (shr->hash, NULL, dl_list, NULL);

	add_id[0] = shr->filename;
	add_id[1] = shr->user;
	add_id[2] = (insert_node) ? "" : "Processing...";

	new_node = gtk_ctree_insert_node (GTK_CTREE (dl_list), insert_node, NULL,
									  add_id, 0, NULL, NULL, NULL, NULL, 0, 0);

	obj_set_data (OBJ_DATA (transfer), "ft_app", NULL);
	obj_set_data (OBJ_DATA (transfer), "tree", dl_list);
	obj_set_data (OBJ_DATA (transfer), "node", new_node);

	SET_NODE_DATA (dl_list, new_node, transfer);

	return TRUE;
}

/*****************************************************************************/

static int download_response (int fd, Transfer *transfer, GData *ft_data)
{
	GtkCTree     *ctree;
	GtkCTreeNode *node;
	char *transfer_info;
	int   difference;

	ctree = GTK_CTREE (obj_get_data (OBJ_DATA (transfer), "tree"));
	node  = GTK_CTREE_NODE (obj_get_data (OBJ_DATA (transfer), "node"));

	/* download has errored out or closed */
	if (fd == DAEMON_EOF)
	{
		/* TODO - check errors */
		printf ("*** transfer finished ... %lu / %lu\n",
				transfer->transmit, transfer->total);

		gtk_ctree_node_set_text (ctree, node, 2, "Finished (maybe?)");

		transfer->finished = TRUE;
		obj_set_data (OBJ_DATA (transfer), "connection", NULL);

		return FALSE;
	}

	difference =
		ATOI (g_datalist_get_data (&ft_data, "RECV")) - transfer->transmit;

	if (difference < 0)
		return TRUE;

	transfer->transmit += difference;
	transfer->total     = ATOI (g_datalist_get_data (&ft_data, "TOTAL"));

	/* prevent FPE */
	if (!transfer->total)
		return TRUE;

	transfer_info =
		g_strdup_printf ("%.02f%% (%lu/%lu) @ %.02fk/s",
						 (((float)transfer->transmit / (float)transfer->total) * 100.0),
						 transfer->transmit, transfer->total,
						 (difference / 1024.0) / STATS_INTERVAL);

	gtk_ctree_node_set_text (ctree, node, 2, transfer_info);

	free (transfer_info);

	return TRUE;
}

/* process the inactive nodes ... */
static void download_start_node (GtkCTree *ctree, GtkCTreeNode *node,
								 void *data)
{
	Transfer *transfer;
	GList *locations = NULL;
	FEConnection *c;

	transfer = gtk_ctree_node_get_row_data (GTK_CTREE (ctree), node);

	assert (transfer);

	if (transfer->active)
	{
		if (transfer->finished)
			fprintf (stderr, "dipshit you've already downloaded this file\n");

		fprintf (stderr, "transfer is active, TODO -- add_source\n");
		return;
	}

	/* we now have an active transfer, let's fuck with it */
	transfer->active = TRANSFER_ACTIVE;

	printf ("activating search for %u %s\n{\n\t%s\n",
			SHARE (transfer)->filesize, SHARE (transfer)->filename,
			SHARE (transfer)->location);

	locations = g_list_prepend (locations, SHARE (transfer)->location);

	if (GTK_CTREE_ROW (node)->children)
	{
		Transfer *ctransfer;
		GtkCTreeNode *cnode;

		cnode = GTK_CTREE_ROW (node)->children;

		for (; cnode; cnode = GTK_CTREE_ROW (cnode)->sibling)
		{
			ctransfer = gtk_ctree_node_get_row_data (GTK_CTREE (ctree), cnode);
			if (!ctransfer)
				return;

			printf ("\t%s\n", SHARE (ctransfer)->location);
			locations = g_list_prepend (locations, SHARE (ctransfer)->location);
		}
	}

#if 0
	c = daemon_request ((ParsePacketFunc) download_response, transfer,
	                    "<transfer method=download action=start size=\"%u\" name=\"%s\"/>\n"
	                    "<transfer method=download action=addsource location=\"%s\"/>\n",
	                    SHARE (transfer)->filesize, SHARE (transfer)->filename,
						SHARE (transfer)->location);
#endif

	printf ("}\n");

	printf ("c = %p\n", c);
	obj_set_data (OBJ_DATA (transfer), "FEConnection", c);

#if 0
	printf ("downloading is not yet supported, try:\n");
	printf ("giFTget %u \"%s\" ",
			SHARE (transfer)->filesize, SHARE (transfer)->filename);

	{
		GList *ptr = locations;

		for (; locations; locations = locations->next)
			printf ("%s ", (char *)locations->data);

		locations = ptr;
	}
	printf ("\n");
#endif

	g_list_free (locations);
}

/* actually start downloading the new files ... */
void download_start (GtkWidget *dl_list)
{
	trace ();

	/* search the root nodes only ... */
	gtk_ctree_post_recursive_to_depth (GTK_CTREE (dl_list), NULL, 1,
									   (GtkCTreeFunc) download_start_node,
									   NULL);
}

/* if cancel is true, the temporary file will be removed and will be removed
 * from the clist.  otherwise, the download will merely be suspended */
void download_stop (Transfer *transfer, int cancel)
{
	GtkCTree     *ctree;
	GtkCTreeNode *node;
	FEConnection   *c;
	char         *cancel_str;

	trace ();

	assert (transfer);

	ctree = GTK_CTREE (obj_get_data (OBJ_DATA (transfer), "tree"));
	node  = GTK_CTREE_NODE (obj_get_data (OBJ_DATA (transfer), "node"));

	if (!(c = obj_get_data (OBJ_DATA (transfer), "FEConnection")))
	{
		printf ("download %p is already inactive\n", transfer);
		return;
	}

	cancel_str = g_strdup_printf ("<transfer method=download action=\"%s\"/>\n",
	                              (cancel ? "close" : "stop"));
	net_send (c->fd, cancel_str);
	free (cancel_str);

	/* connect_close (c); */
	/* obj_set_data (OBJ_DATA (transfer), "FEConnection", NULL); */

	if (cancel)
		gtk_ctree_remove_node (GTK_CTREE (ctree), node);
}

/*****************************************************************************/

static void menu_pause_download (GtkWidget *menu, Transfer *transfer)
{
	download_stop (transfer, FALSE);
}

static void menu_cancel_download (GtkWidget *menu, Transfer *transfer)
{
	download_stop (transfer, TRUE);
}

int menu_popup_download (GtkWidget *dl_list, Transfer *download, guint button,
						 guint32 at)
{
	GtkWidget *menu;
	GtkWidget *item;

	menu = gtk_menu_new ();

	item = menu_append (menu, "Pause Download");
	gtk_signal_connect (GTK_OBJECT (item), "activate",
						GTK_SIGNAL_FUNC (menu_pause_download), download);

	item = menu_append (menu, "Cancel Download");
	gtk_signal_connect (GTK_OBJECT (item), "activate",
						GTK_SIGNAL_FUNC (menu_cancel_download), download);

	gtk_widget_show_all (menu);
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, at);

	return TRUE;
}
